/*-----------------------------------------------------------------------/
/  Low level disk interface modlue include file   (C)ChaN, 2013          /
/-----------------------------------------------------------------------*/


/**
 * \file diskio.h
 * \brief Definitions for SD I/O -> SPI Functions and Error-Codes
 * \author <b>generated by:</b> ChaN
 * \author <b>implemented & edited by:</b> Dorothee Brunner, dorothee.brunner@epcos.com
 *
 * All Functions in mmc.c are written for single drive, that means only one SD Card and no other medium is used
 * project: EVALUATIONKIT
 *
 * \date 12.08.2014 - first implementation
 * \date 13.08.2014 - create documentation
 */

#ifndef _DISKIO_DEFINED
#define _DISKIO_DEFINED

#ifdef __cplusplus
	extern "C" {
#endif

#define _USE_WRITE	1	/** 1: Enable disk_write function */
#define _USE_IOCTL	1	/** 1: Enable disk_ioctl function */

#include "integer.h"


/** Status of Disk Functions */
typedef BYTE	DSTATUS;

/** Results of Disk Functions */
typedef enum {
	RES_OK = 0,		/**< 0: Successful */
	RES_ERROR,		/**< 1: R/W Error */
	RES_WRPRT,		/**< 2: Write Protected */
	RES_NOTRDY,		/**< 3: Not Ready */
	RES_PARERR		/**< 4: Invalid Parameter */
} DRESULT;


/*---------------------------------------*/
/* Prototypes for disk control functions */

/**
* \brief initialization of SD Card for SPI mode
* \param[in] pdrv  Physical drive number to identify the target device.
* \param[out] DSTATUS disk status
* \return STA_NOINIT Indicates that the device is not initialized.
* \return STA_NODISK Indicates that no medium in the drive.
* \return STA_PROTECT Indicates that the medium is write protected.
*
* This function initializes a storage device and put it ready to generic read/write data. When the function succeeded, STA_NOINIT flag in the return value is cleared.
* It brings the SD Card into SPI mode. The initalization follows the following flow diagram 
* \image html sdinit.png
*
** \attention the function is only implemented for single-drive. If more consumers are required the function needs a change in mmc.c

* \date 12.08.2014 - First implementation
*
* \todo check if the code really follows the flow diagram
* \bug initialization failed
*
* \version 1.0
* \author <b>generated by:</b> clawson, avrfreaks.net
* \author <b>implemented & edited by:</b> Dorothee Brunner, dorothee.brunner@epcos.com
**/
DSTATUS disk_initialize (BYTE pdrv);
/**
* \brief gives SD status
* \param[in] pdrv  Physical drive number to identify the target device.
* \param[out] DSTATUS disk status
* \return STA_NOINIT Indicates that the device is not initialized.
* \return STA_NODISK Indicates that no medium in the drive.
* \return STA_PROTECT Indicates that the medium is write protected.
*
* This function gives one of the four status' of the SD Card back. The Working Status is !STA_NOINIT => 0<br>
* The function is only usefull for troubleshooting
* \attention there is <b>no read-out</b> of the status implemented! <br>
* \attention the function is only implemented for single-drive. If more consumers are required the function needs a change in mmc.c
*
* \date 12.08.2014 - First implementation
*
* \version 1.0
* \author <b>generated by:</b> clawson, avrfreaks.net
* \author <b>implemented & edited by:</b> Dorothee Brunner, dorothee.brunner@epcos.com
**/
DSTATUS disk_status (BYTE pdrv);


DRESULT disk_read (BYTE pdrv, BYTE* buff, DWORD sector, UINT count);
DRESULT disk_write (BYTE pdrv, const BYTE* buff, DWORD sector, UINT count);
DRESULT disk_ioctl (BYTE pdrv, BYTE cmd, void* buff);

void send_byte (char dat);
void power_off (void);
void power_on (void);

/* Disk Status Bits (DSTATUS) */

#define STA_NOINIT		0x01	/**< Drive not initialized */
#define STA_NODISK		0x02	/**< No medium in the drive */
#define STA_PROTECT		0x04	/**< Write protected */


/* Command code for disk_ioctrl fucntion */

/* Generic command (used by FatFs) */
#define CTRL_SYNC			0	/* Flush disk cache (for write functions) */
#define GET_SECTOR_COUNT	1	/* Get media size (for only f_mkfs()) */
#define GET_SECTOR_SIZE		2	/* Get sector size (for multiple sector size (_MAX_SS >= 1024)) */
#define GET_BLOCK_SIZE		3	/* Get erase block size (for only f_mkfs()) */
#define CTRL_ERASE_SECTOR	4	/* Force erased a block of sectors (for only _USE_ERASE) */

/* Generic command (not used by FatFs) */
#define CTRL_POWER			5	/* Get/Set power status */
#define CTRL_LOCK			6	/* Lock/Unlock media removal */
#define CTRL_EJECT			7	/* Eject media */
#define CTRL_FORMAT			8	/* Create physical format on the media */

/* MMC/SDC specific ioctl command */
#define MMC_GET_TYPE		10	/* Get card type */
#define MMC_GET_CSD			11	/* Get CSD */
#define MMC_GET_CID			12	/* Get CID */
#define MMC_GET_OCR			13	/* Get OCR */
#define MMC_GET_SDSTAT		14	/* Get SD status */

/* ATA/CF specific ioctl command */
#define ATA_GET_REV			20	/* Get F/W revision */
#define ATA_GET_MODEL		21	/* Get model name */
#define ATA_GET_SN			22	/* Get serial number */

#ifdef __cplusplus
	}
#endif

#endif
